--- old/usr/src/cmd/sgs/rtld/common/malloc.c	2014-06-11 19:45:18.091917689 -0400
+++ new/usr/src/cmd/sgs/rtld/common/malloc.c	2014-06-11 19:45:18.115269176 -0400
@@ -207,11 +207,23 @@
 calloc(size_t num, size_t size)
 {
 	void *	mp;
+	size_t	total;
 
-	num *= size;
-	if ((mp = malloc(num)) == NULL)
+	if (num == 0 || size == 0) {
+		total = 0;
+	} else {
+		total = num * size;
+
+		/* check for overflow */
+		if (total / num != size) {
+			errno = ENOMEM;
+			return (NULL);
+		}
+	}
+
+	if ((mp = malloc(total)) == NULL)
 		return (NULL);
-	(void) memset(mp, 0, num);
+	(void) memset(mp, 0, total);
 	return (mp);
 }
 
--- old/usr/src/lib/libmalloc/common/malloc.c	2014-06-11 19:45:19.057409112 -0400
+++ new/usr/src/lib/libmalloc/common/malloc.c	2014-06-11 19:45:19.070698644 -0400
@@ -27,8 +27,6 @@
 /*	Copyright (c) 1988 AT&T	*/
 /*	  All Rights Reserved  	*/
 
-#pragma ident	"%Z%%M%	%I%	%E% SMI"
-
 #include <sys/types.h>
 
 #ifndef debug
@@ -37,6 +35,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include "assert.h"
 #include "malloc.h"
 #include "mallint.h"
@@ -845,12 +844,24 @@
 calloc(size_t num, size_t size)
 {
 	char *mp;
+	size_t total;
+
+	if (num == 0 || size == 0) {
+		total = 0;
+	} else {
+		total = num * size;
+
+		/* check for overflow */
+		if ((total / num) != size) {
+			errno = ENOMEM;
+			return (0);
+		}
+	}
 
-	num *= size;
-	mp = malloc(num);
+	mp = malloc(total);
 	if (mp == NULL)
 		return (NULL);
-	(void) memset(mp, 0, num);
+	(void) memset(mp, 0, total);
 	return (mp);
 }
 
--- old/usr/src/lib/libmapmalloc/common/calloc.c	2014-06-11 19:45:19.553468347 -0400
+++ new/usr/src/lib/libmapmalloc/common/calloc.c	2014-06-11 19:45:19.566750903 -0400
@@ -24,10 +24,10 @@
  * Use is subject to license terms.
  */
 
-#pragma ident	"%Z%%M%	%I%	%E% SMI"
-
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
+
 /*
  * calloc - allocate and clear memory block
  */
@@ -36,12 +36,24 @@
 calloc(size_t num, size_t size)
 {
 	void *mp;
+	size_t total;
+
+	if (num == 0 || size == 0) {
+		total = 0;
+	} else {
+		total = num * size;
+
+		/* check for overflow */
+		if (total / num != size) {
+			errno = ENOMEM;
+			return (NULL);
+		}
+	}
 
-	num *= size;
-	mp = malloc(num);
+	mp = malloc(total);
 	if (mp == NULL)
 		return (NULL);
-	(void) memset(mp, 0, num);
+	(void) memset(mp, 0, total);
 	return (mp);
 }
 
--- old/usr/src/lib/libmtmalloc/common/mtmalloc.c	2014-06-11 19:45:20.001305666 -0400
+++ new/usr/src/lib/libmtmalloc/common/mtmalloc.c	2014-06-11 19:45:20.016616962 -0400
@@ -331,7 +331,19 @@
 calloc(size_t nelem, size_t bytes)
 {
 	void * ptr;
-	size_t size = nelem * bytes;
+	size_t size;
+
+	if (nelem == 0 || bytes == 0) {
+		size = 0;
+	} else {
+		size = nelem * bytes;
+
+		/* check for overflow */
+		if (size / nelem != bytes) {
+			errno = ENOMEM;
+			return (NULL);
+		}
+	}
 
 	ptr = malloc(size);
 	if (ptr == NULL)
